library(rjags)

## read data
ph<-read.table("performance.txt",header=TRUE)
ph<-ph[ph$family!="?",]

## format survival data
## gla = 0, sla = 1; ms = 0, ac = 1
data<-list(y=as.numeric(ph$survivorship),pop=as.numeric(ph$pop)-1,plant=2-as.numeric(ph$plant),fam=as.numeric(as.character(ph$family)),n=dim(ph)[1],nfam=51)

## specify jags model
jagsModel="
    model {
    for (i in 1:n){
        y[i] ~ dbern(p[i])
        logit(p[i])<-beta[1] + beta[2]*pop[i] + beta[3]*plant[i] + beta[4]*pop[i]*plant[i] + alpha0[fam[i]] * (1 - pop[i]) * (1 - plant[i])+ alpha1[fam[i]] * pop[i] * (1 - plant[i]) + gamma0[fam[i]] * (1 - pop[i]) * plant[i] + gamma1[fam[i]] * pop[i] * plant[i]
    }
    
    ## priors for regression coefficients
    for (k in 1:4) {
        beta[k] ~ dnorm(0,0.001)    
    }
    
    ## random effect prior
    for (j in 1:(nfam-1)){
        alpha0[j] ~ dnorm(0,tau0)
        alpha1[j] ~ dnorm(0,tau0)
        gamma0[j] ~ dnorm(0,tau0)
        gamma1[j] ~ dnorm(0,tau0)
    }
    alpha0[nfam]<--1*sum(alpha0[1:(nfam-1)])
    alpha1[nfam]<--1*sum(alpha1[1:(nfam-1)])
    gamma0[nfam]<--1*sum(gamma0[1:(nfam-1)])
    gamma1[nfam]<--1*sum(gamma1[1:(nfam-1)])

    ## precision priors
    tau0 ~ dgamma(1,0.01)
    tau1 ~ dgamma(1,0.01)
    tau2 ~ dgamma(1,0.01)
    tau3 ~ dgamma(1,0.01)

    sd<-sqrt(1/tau0)

    ## monitor expected values, and alpha sum
    ex[1]<-1/(1 + exp(-1 * beta[1])) ## gla on ms
    ex[2]<-1/(1 + exp(-1 * (beta[1] + beta[3]))) ## gla on ac
    ex[3]<-1/(1 + exp(-1 * (beta[1] + beta[2]))) ## sla on ms
    ex[4]<-1/(1 + exp(-1 * (beta[1] + beta[2] + beta[3] + beta[4]))) ## sla on ac
    sa0<-sum(alpha0)
    sa1<-sum(alpha1)
 
    }  
"

## fit model
model<-jags.model(textConnection(jagsModel),data=data,n.chains=3)
update(model,n.iter=5000)
mcmcSamples<-coda.samples(model=model,variable.names=c("beta","sa0","sa1","ex","sd"),n.iter=10000,thin=2)

## summarize and simple plot median and 95% ETPI, population, plant, and pxp significant
out<-summary(mcmcSamples)
##plot(1:4,out$quantiles[5:8,3],ylim=c(0,1))
##segments(1:4,out$quantiles[5:8,1],1:4,out$quantiles[5:8,5])

## format weight data
## gla = 0, sla = 1; ms = 0, ac = 1
x<-which(as.numeric(ph$survivorship)==1) ## retain individuals that lived
data<-list(y=as.numeric(ph$weight[x]),pop=as.numeric(ph$pop)[x]-1,plant=2-as.numeric(ph$plant)[x],fam=as.numeric(as.character(ph$family))[x],n=length(x),nfam=51)

## specify jags model
jagsModel="
    model {
    for (i in 1:n){
        y[i] ~ dnorm(alpha[fam[i]],tau)
        mu[i]<-beta[1] + beta[2]*pop[i] + beta[3]*plant[i] + beta[4]*pop[i]*plant[i] + alpha0[fam[i]] * (1 - pop[i]) * (1 - plant[i])+ alpha1[fam[i]] * pop[i] * (1 - plant[i]) + gamma0[fam[i]] * (1 - pop[i]) * plant[i] + gamma1[fam[i]] * pop[i] * plant[i]
    }
    
    ## priors for regression coefficients
    for (k in 1:4) {
        beta[k] ~ dnorm(0,0.001)    
    }

    ## random effect prior
    for (j in 1:(nfam-1)){

	alpha0[j] ~ dnorm(0,tau0)
        alpha1[j] ~ dnorm(0,tau0)
	gamma0[j] ~ dnorm(0,tau0)
	gamma1[j] ~ dnorm(0,tau0)

    }
    alpha0[nfam]<--1*sum(alpha0[1:(nfam-1)])
    alpha1[nfam]<--1*sum(alpha1[1:(nfam-1)])
    gamma0[nfam]<--1*sum(gamma0[1:(nfam-1)])
    gamma1[nfam]<--1*sum(gamma1[1:(nfam-1)])

    ## precision priors
    tau ~ dgamma(1,0.01)
    tau0 ~ dgamma(1,0.01)
    tau1 ~ dgamma(1,0.01)
    tau2 ~ dgamma(1,0.01)
    tau3 ~ dgamma(1,0.01)


    sd<-sqrt(1/tau0)


    ## monitor expected values, and alpha sum
    ex[1]<-beta[1] ## gla on ms
    ex[2]<-beta[1] + beta[3]## gla on ac
    ex[3]<-beta[1] + beta[2] ## sla on ms
    ex[4]<-beta[1] + beta[2] + beta[3] + beta[4] ## sla on ac
    sa0<-sum(alpha0)
    sa1<-sum(alpha1)
 
    }  
"
## with sweeping
jagsModel="
    model {
    for (i in 1:n){
        y[i] ~ dnorm(alpha[i],tau)
	alpha[i]<-alpha00[fam[i]] * (1 - pop[i]) * (1 - plant[i]) + alpha10[fam[i]] * pop[i] * (1 - plant[i]) + alpha01[fam[i]] * (1 - pop[i]) * plant[i] + alpha11[fam[i]] * pop[i] * plant[i]
    }

    ## means for different pop x plant combinations
    mu[1]<-beta[1] + beta[2]*0 + beta[3]*0 + beta[4]*0*0
    mu[2]<-beta[1] + beta[2]*1 + beta[3]*0 + beta[4]*1*0
    mu[3]<-beta[1] + beta[2]*0 + beta[3]*1 + beta[4]*0*1
    mu[4]<-beta[1] + beta[2]*1 + beta[3]*1 + beta[4]*1*1
    
    ## priors for regression coefficients
    for (k in 1:4) {
        beta[k] ~ dnorm(0,0.001)    
    }

    ## random effect prior
    for (j in 1:nfam){
	alpha00[j] ~ dnorm(mu[1],tauA)
    alpha10[j] ~ dnorm(mu[2],tauA)
	alpha01[j] ~ dnorm(mu[3],tauA)
	alpha11[j] ~ dnorm(mu[4],tauA)
    }
    
    ## precision priors
    tau ~ dgamma(1,0.01)
    tauA ~ dgamma(1,0.01)


    sd<-sqrt(1/tauA)


    ## monitor expected values, and alpha sum
    ex[1]<-beta[1] ## gla on ms
    ex[2]<-beta[1] + beta[3]## gla on ac
    ex[3]<-beta[1] + beta[2] ## sla on ms
    ex[4]<-beta[1] + beta[2] + beta[3] + beta[4] ## sla on ac
 
    }  
"

## fit model
model<-jags.model(textConnection(jagsModel),data=data,n.chains=3)
update(model,n.iter=5000)
mcmcSamplesWgt<-coda.samples(model=model,variable.names=c("beta","ex","sd"),n.iter=10000,thin=2)
#mcmcSamplesWgt<-coda.samples(model=model,variable.names=c("beta","sa0","sa1","ex","sd"),n.iter=10000,thin=2)

## summarize and simple plot median and 95% ETPI, population, plant, and pxp significant
outWgt<-summary(mcmcSamplesWgt)
plot(1:4,outWgt$quantiles[5:8,3],ylim=c(0,1))
segments(1:4,outWgt$quantiles[5:8,1],1:4,outWgt$quantiles[5:8,5])

postscript("experiment.eps",width=4,height=8)
par(mfrow=c(2,1))
par(pty='s')
par(mar=c(4.5,5,1,0.5))
# survival
plot(c(0.15,0.85),out$statistics[c(5:6),1],ylim=c(0,1),xlim=c(0,1),type='b',lty=2,lwd=1.5,pch=16,axes=FALSE,xlab="",ylab="survival probability",cex.lab=1.5,cex=1.1)
points(c(0.15,0.85),out$statistics[c(7:8),1],type='b',lty=2,lwd=1.5,pch=23,cex=1.1)
lb<-out$statistics[c(5:8),1]-out$statistics[c(5:8),2]
ub<-out$statistics[c(5:8),1]+out$statistics[c(5:8),2]
segments(c(0.15,0.85,0.15,0.85),lb,c(0.15,0.85,0.15,0.85),ub,lwd=2)
axis(1,at=c(0.15,0.85),c("Medicago","Astragalus"),cex.axis=1.5)
axis(2,cex.axis=1.3)
box()
legend(0.05,0.95,c("GLA","SLA"),pch=c(16,23),bty='n',cex=1.1)
# weight
plot(c(0.15,0.85),outWgt$statistics[c(5:6),1],ylim=c(2,9),xlim=c(0,1),type='b',lty=2,lwd=1.5,pch=16,axes=FALSE,xlab="plant treatment",ylab="adult weight (grams)",cex.lab=1.5,cex=1.1)
points(c(0.15,0.85),outWgt$statistics[c(7:8),1],type='b',lty=2,lwd=1.5,pch=23,cex=1.1)
lb<-outWgt$statistics[c(5:8),1]-outWgt$statistics[c(5:8),2]
ub<-outWgt$statistics[c(5:8),1]+outWgt$statistics[c(5:8),2]
segments(c(0.15,0.85,0.15,0.85),lb,c(0.15,0.85,0.15,0.85),ub,lwd=2)
axis(1,at=c(0.15,0.85),c("Medicago","Astragalus"),cex.axis=1.5)
axis(2,cex.axis=1.3)
box()
#legend(0.05,0.95,c("GLA","SLA"),pch=c(16,23),bty='n',cex=1.1)
dev.off()

## barplot
pdf("experiment.pdf",width=4,height=8)
par(mfrow=c(2,1))
par(pty='s')
par(mar=c(4.5,5,1,0.5))
# survival
bout<-barplot(cbind(out$statistics[c(5,7),1],out$statistics[c(6,8),1]),beside=TRUE,xlab="",ylab="survival probability",names.arg=c("M. sativa","A. canadensis"),cex.lab=1.5,cex.names=1.3,ylim=c(0,1),legend.text=c("GLA","SLA"),args.legend=list(x=3))

lb<-out$statistics[c(5,7,6,8),1]-out$statistics[c(5,7,6,8),2]
ub<-out$statistics[c(5,7,6,8),1]+out$statistics[c(5,7,6,8),2]
segments(as.vector(bout),lb,as.vector(bout),ub,lwd=2)

#weigth 
bout<-barplot(cbind(outWgt$statistics[c(5,7),1],outWgt$statistics[c(6,8),1]),beside=TRUE,xlab="plant treatment",ylab="adult weight (grams)",names.arg=c("M. sativa","A. canadensis"),cex.lab=1.5,cex.names=1.3,ylim=c(0,10))

lb<-outWgt$statistics[c(5,7,6,8),1]-outWgt$statistics[c(5,7,6,8),2]
ub<-outWgt$statistics[c(5,7,6,8),1]+outWgt$statistics[c(5,7,6,8),2]
segments(as.vector(bout),lb,as.vector(bout),ub,lwd=2)


#legend(0.05,0.95,c("GLA","SLA"),pch=c(16,23),bty='n',cex=1.1)
dev.off()


