#include <iostream>
#include <sstream>
#include <fstream>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_cdf.h>
#include <float.h>

#include <math.h>
#include "hdf5.h"

#include "popmod.H"

using namespace std;

void usage(char * name){
  fprintf(stderr,"\n%s version %s\n\n", name, VERSION); 
  fprintf(stderr, "Usage: popmod -i infile.txt [options]\n");
  fprintf(stderr, "-i Infile with genetic data for the population\n");
  fprintf(stderr, "-n Number of MCMC steps for the analysis [default = 10000]\n");
  fprintf(stderr, "-b Discard the first n MCMC samples as a burn-in [default = 1000]\n");
  fprintf(stderr, "-t Thin MCMC samples by recording every nth value [default = 2]\n");
  fprintf(stderr, "-o HDF5 format outfile with .hdf5 suffix [default = mcmcout.hdf5]\n");
  fprintf(stderr, "-a Parameter for beta prior on allele frequencies [default = 0.5]\n");
  exit(1);
}

// function to read the genotype likelihoods
void readData(string infile, datacont * data){
  int i, j, k;
  ifstream file;
  istringstream stream;
  string line, element;
  double genotypes[NGEN];
  double gensum = 0;

  // open genotype likelihood file
  file.open(infile.c_str());
  if (!file){
    cerr << "Cannot open file " << infile << endl;
    exit(1);
  }

  // read line with number of loci and individuals
  getline(file, line);
  stream.str(line);
  stream.clear();
  stream >> element; // number of individuals
  data->nind = atoi(element.c_str());  
  stream >> element; // number of loci
  data->nloci = atoi(element.c_str()); 

  cerr << "Number of loci: " << data->nloci << endl;
  cerr << "Number of individuals: " << data->nind << endl;

  // allocate memory for data
  data->gl = new gsl_matrix * [data->nloci];
  for(i=0; i<data->nloci; i++){
    data->gl[i] = gsl_matrix_calloc(data->nind, NGEN);
  }
  data->pincl = gsl_vector_int_calloc(data->nind);

  // read data
  getline(file, line); // individual ids, these are not retained

  getline(file, line); // contains binary indicator, include for allele freq.
  stream.str(line);
  stream.clear();
  for(j=0; j<data->nind; j++){
    stream >> element;
    gsl_vector_int_set(data->pincl, j, atoi(element.c_str()));
  }

  // genotype likelihood lines, this are phred-scaled
  for(i=0; i<data->nloci; i++){
    getline(file, line); // data for one locus
    stream.str(line);
    stream.clear();
    stream >> element; // locus id, this is not retained
    for(j=0; j<data->nind; j++){
      gensum = 0;
      for(k=0; k<NGEN; k++){ // store genotype likelihoods
	stream >> element; // need to convert from phred scale: phred
	// = -10 log10(prob), prob = 10^(phred/-10)
	genotypes[k] = pow(10, (atof(element.c_str())/-10.0));
	gensum += genotypes[k];
      }
      for(k=0; k<NGEN; k++){ // normalize genotype likelihoods
	genotypes[k] = genotypes[k]/gensum;
	if (genotypes[k] == 0) // set to DBL_MIN if 0
	  genotypes[k] = DBL_MIN;
	gsl_matrix_set(data->gl[i], j, k, genotypes[k]);
      }
    }
  } 
}

// function to create the hdf5 objects and outfile
void createHdf5(hdf5cont * hdf5, datacont * data, int N){
  /* HDF5, create datasets */
  /*
   * Describe the size of the array and create the data space for
   * fixed size dataset.  I reuse dataspace and datatype, because they
   * are the same for each of the parameters.
   */
  // set dimensions
  hdf5->dimsLIG[0] = data->nloci; 
  hdf5->dimsLIG[1] = data->nind; 
  hdf5->dimsLIG[2] = NGEN; 

  hdf5->dimsLM[0] = data->nloci; 
  hdf5->dimsLM[1] = N; 

  // create dataspaces
  hdf5->dataspaceLIG = H5Screate_simple(3,  hdf5->dimsLIG, NULL); /* RANK is 3 */
  hdf5->dataspaceLM = H5Screate_simple(2,  hdf5->dimsLM, NULL); /* RANK is 2 */

  // define datatype by copying an existing datatype, little endian float
  hdf5->datatype = H5Tcopy(H5T_NATIVE_FLOAT); 
  hdf5->status = H5Tset_order(hdf5->datatype, H5T_ORDER_LE);

  // create datasets
  hdf5->datasetG = H5Dcreate2(hdf5->file, "g", hdf5->datatype, hdf5->dataspaceLIG, 
				  H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

  hdf5->datasetP = H5Dcreate2(hdf5->file, "p", hdf5->datatype, hdf5->dataspaceLM, 
			      H5P_DEFAULT, H5P_DEFAULT, H5P_DEFAULT);

  hdf5->locbuffer = H5Screate_simple(1, &hdf5->dimsLIG[0], NULL); 
}


// function initializes parameters, genotype uses gibbs update
void initParams(datacont * data, paramcont * params){
  int i, j;
  double g, y, n;
  int m = 2; // ploidy
  double p;

  // allocate memory
  params->g = gsl_matrix_int_calloc(data->nloci, data->nind);
  params->p = gsl_vector_calloc(data->nloci);
  params->postg = new gsl_matrix * [data->nloci];
  for(i=0; i<data->nloci; i++){
    params->postg[i] = gsl_matrix_calloc(data->nind, NGEN);
  }
  params->aux = gsl_vector_calloc(data->nloci);

  // sample initial allele frequencies based on genotype likelihoods
  for(i=0; i<data->nloci; i++){
    y = 1; // minimum values for sampling
    n = 1;
    for(j=0; j<data->nind; j++){
      if(gsl_vector_int_get(data->pincl, j) == 1){ // include this ind. in allele freq. update
	// expected count of the reference allele from genotype likelihood
	g = 2 * gsl_matrix_get(data->gl[i], j, 0) + gsl_matrix_get(data->gl[i], j, 1); 
	y += g;
	n += (m - g);
      }
    }
    p = gsl_ran_beta(r, y, n);
    gsl_vector_set(params->p, i, p);
  }
  updateG(data, params);
}


// function that controls MCMC updates
void mcmcUpdate(datacont * data, paramcont * params){
  updateP(data, params);
  updateG(data, params);
}

// function implements gibbs update of allele frequencies (P)
void updateP(datacont * data, paramcont * params){
  int i, j;
  double y, n;
  int m = 2; // ploidy
  int g;
  double p;

  // update one locus at a time
  for(i=0; i<data->nloci; i++){
    // apply prior
    y = params->pprior;
    n = params->pprior;
    for(j=0; j<data->nind; j++){
      if(gsl_vector_int_get(data->pincl, j) == 1){ // include this ind. in allele freq. update
	g = gsl_matrix_int_get(params->g, i, j); // increment sum from genotypes
	y += g;
	n += (m - g);
      }
    }
    // gibbs sample of p from the full conditional
    p = gsl_ran_beta(r, y, n);
    gsl_vector_set(params->p, i, p);
  }
}

// function implements gibbs update of genotypes
void updateG(datacont * data, paramcont * params){
  int i, j, k;
  int m = 2; //  ploidy
  double p;
  double gprior[NGEN];
  double gprob[NGEN];
  unsigned int gsam[NGEN];

  // update one locus at a time
  for(i=0; i<data->nloci; i++){
    p = gsl_vector_get(params->p, i);
    //calculate prior probs. under HWE
    gprior[0] = gsl_pow_2(p); // Hom. Ref
    gprior[1] = 2.0 * p * (1 - p); // Het
    gprior[2] = gsl_pow_2((1.0-p)); // Hom. Alt
    //update one individual at a time
    for(j=0; j<data->nind; j++){
      // calculate and sample G based on full conditional
      for(k=0; k<NGEN; k++){
	gprob[k] = gprior[k] * gsl_matrix_get(data->gl[i], j, k); // prior x likelihood
      }
      gsl_ran_multinomial(r, NGEN, 1, gprob, gsam);
      for(k=0; k<NGEN; k++){
	if(gsam[k] == 1){ // found the sampled genotype, set (reverse polarity), and break
	  gsl_matrix_int_set(params->g, i, j, (m - k));
	  break;
	}
      }
    }
  }
}

// function writes samples from current MCMC step
void writeStep(hdf5cont * hdf5, datacont * data, paramcont * params, int s){
  
  // model only includes allele frequencies 

  // select hyperslab
  hdf5->sr2[0] = 0;
  hdf5->sr2[1] = s; // current step
  hdf5->br2[0] = data->nloci;
  hdf5->br2[1] = 1;
  hdf5->cr2[0] = 1;
  hdf5->cr2[1] = 1;

  hdf5->status = H5Sselect_hyperslab(hdf5->dataspaceLM, H5S_SELECT_SET, hdf5->sr2, NULL, hdf5->cr2, hdf5->br2); 

  // load buffer and write
  hdf5->status = H5Dwrite(hdf5->datasetP, H5T_NATIVE_DOUBLE, hdf5->locbuffer, hdf5->dataspaceLM, H5P_DEFAULT, params->p->data);
}

// function updates the genotype posterior probability distribution based on sampled genotypes
void updatePostG(datacont * data, paramcont * params, int N){
  int i, j;
  int m = 2; // ploidy
  int g;
  double gpr;

  for(i=0; i<data->nloci; i++){
    for(j=0; j<data->nind; j++){
      g = gsl_matrix_int_get(params->g, i, j);
      gpr = (double) 1.0/N; // this is the amount to add to the posterio
      // m - g gives the genotype the index, because g counts reference alleles
      gsl_matrix_set(params->postg[i], j, (m - g), gpr + gsl_matrix_get(params->postg[i], j, (m - g)));
    }
  }
}
      
// function writes the genotype posterior probability distribution
void writePostG(hdf5cont * hdf5, datacont * data, paramcont * params){
  int i, j, k;

  // select hyperslab for genotype
  hdf5->sr3[0] = 0;
  hdf5->br3[0] = data->nloci;
  hdf5->br3[1] = 1;
  hdf5->br3[2] = 1;
  hdf5->cr3[0] = 1;
  hdf5->cr3[1] = 1;
  hdf5->cr3[2] = 1;

  for(j=0; j<data->nind; j++){
    for(k=0; k<NGEN; k++){
      hdf5->sr3[1] = j;
      hdf5->sr3[2] = k;
      
      hdf5->status = H5Sselect_hyperslab(hdf5->dataspaceLIG, H5S_SELECT_SET, hdf5->sr3, NULL, hdf5->cr3, hdf5->br3); 

      // load buffer and write, all loci, one individual and genotype
      for(i=0; i<data->nloci; i++){
	gsl_vector_set(params->aux, i, gsl_matrix_get(params->postg[i], j, k));
      }
      hdf5->status = H5Dwrite(hdf5->datasetG, H5T_NATIVE_DOUBLE, hdf5->locbuffer, hdf5->dataspaceLIG, H5P_DEFAULT, params->aux->data);
    }
  }
}
