// file: main.C

// popmod - a program to infer genotypes and allele frequencies from
// pre-calculated genotype likelihoods

// Time-stamp: <Tuesday, 12 November 2013, 13:37 MST -- zgompert>

#include <cmath>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_cdf.h>
#include <time.h>
#include <getopt.h>
#include "hdf5.h"
#include "popmod.H"

using namespace std;

gsl_rng * r;  /* global state variable for random number generator */

/* ----------------- */
/* beginning of main */
/* ----------------- */

int main(int argc, char *argv[]) {

  time_t start = time(NULL);
  time_t end;
  int rng_seed = 0, ch = 0, step = 0, nsteps = 10000;
  int thin = 2, burn = 1000;

  string infile = "undefined";
  paramcont params;
  datacont data;
  hdf5cont hdf5;

  params.pprior = 0.5; // Jeffery's prior on allele frequencies

  /* Create a new file using H5F_ACC_TRUNC access, default file
     creation properties, and default file access properties. */
  char * hdf5outfile =  (char *) "mcmcout.hdf5";

  // get command line arguments
  if (argc < 2) {
    usage(argv[0]);
  }
  
  while ((ch = getopt(argc, argv, "b:n:i:t:o:a:")) != -1){
    switch(ch){
    case 'b':
      burn = atoi(optarg);
      break;
    case 'n':
      nsteps = atoi(optarg);
      break;
    case 'i':
      infile = optarg;
      break;
    case 't':
      thin = atoi(optarg);
      break;
    case 'o':
      hdf5outfile = optarg;
      break;
    case 'a':
      params.pprior = atof(optarg);
      break;
     case '?':
    default:
      usage(argv[0]);
    }
  }

  // set up gsl random number generation 
  gsl_rng_env_setup();
  r = gsl_rng_alloc(gsl_rng_default);
  srand(time(NULL));
  rng_seed = rand();
  gsl_rng_set(r, rng_seed); /* seed gsl_rng with output of rand, which
			       was seeded with result of time(NULL) */

  // read genotype likelihood formatted data
  cerr << "Reading data from " << infile << endl;
  readData(infile, &data);

  // create Hdf5 file and data objects
  hdf5.file = H5Fcreate(hdf5outfile, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT);
  createHdf5(&hdf5, &data, ((nsteps - burn) / thin));   

  // sample starting values for MCMC
  cerr << "Generating starting values for MCMC" << endl;
  initParams(&data, &params);

  // main loop for MCMC implementation of allele frequency and genotype model
  cerr << "Running chain " << endl;
  for(step=0; step<nsteps; step++){
    mcmcUpdate(&data, &params); // sample new parameter values
    if((step >= burn) && (((step-burn) % thin) == 0)){
      // write MCMC samples for current step
      writeStep(&hdf5, &data, &params, (int) ((step - burn) / thin)); 
      // running sum of G for posterior distn.
      updatePostG(&data, &params, ((nsteps - burn) / thin));
      cerr << "writing samples from step " << step << endl;
    }
  }
  cerr << endl << "Writing final results" << endl;
  writePostG(&hdf5, &data, &params);
  // close output file
  H5Fclose(hdf5.file);

  // prints run time
  end = time(NULL);
  cerr << "Runtime: " << (end-start)/3600 << " hr " << (end-start)%3600/60 << " min ";
  cerr << (end-start)%60 << " sec" << endl;
  return 0;
}
